/*
** Copyright (C) 2006 Thadeu Lima de Souza Cascardo <cascardo@minaslivre.org>
** Copyright (C) 2009 Thadeu Lima de Souza Cascardo <cascardo@holoscopio.com>
**  
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**  
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**  
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
**  
*/

#include <glib.h>
#include <gnet.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gnutls/gnutls.h>
#include "log.h"
#include "nethook.h"
#include "null.h"
#include "ssl.h"
#include "pop.h"

#define CONFFILE SYSCONFDIR "/popproxy.conf"

void new_client (GServer* server, GConn* conn, gpointer data)
{
  net_hook_t* hook;
  if (conn == NULL)
    {
      g_critical ("Server has received an error event.");
      return;
    }
  g_message ("Received connection from %s.", conn->hostname);
  hook = ssl_hook_new (conn, data);
  pop_hook_new (hook);
  gnet_conn_read (conn);
}

static gchar* configfile;

static GOptionEntry opt_entries[] =
  {
    { "config-file", 'c', 0, G_OPTION_ARG_FILENAME, &configfile,
      "Configuration file location", "file" },
    { NULL }
  };

int main (int argc, char** argv)
{

  GOptionContext* opt_ctx;
  GKeyFile *keyfile;
  GInetAddr* inetaddr;
  GError *error;
  gchar* conf_address;
  gint port;
  gchar *server_address;

  gnutls_global_init ();
  gnet_init ();
  pop_log_init ();

  configfile = CONFFILE;
  opt_ctx = g_option_context_new ("");
  g_option_context_add_main_entries (opt_ctx, opt_entries, NULL);
  if (!g_option_context_parse (opt_ctx, &argc, &argv, NULL))
    {
      g_log (NULL, G_LOG_LEVEL_WARNING | G_LOG_FLAG_FATAL,
	     "Could not parse command line options.");
    }
  g_option_context_free (opt_ctx);
  
  keyfile =  g_key_file_new ();

  error = NULL;
  if (g_key_file_load_from_file (keyfile, configfile,
                                 G_KEY_FILE_NONE, &error) == FALSE)
    {
      fprintf (stderr, "Could not load configuration file %s: %s.\n",
               configfile, error->message);
      g_error_free (error);
      exit (1);
    }

  error = NULL;
  conf_address = g_key_file_get_string (keyfile, "global", "address",
                                        &error);
  if (conf_address == NULL && error != NULL)
    {
      conf_address = g_strdup ("0.0.0.0");
      g_error_free (error);
    }
  error = NULL;
  port = g_key_file_get_integer (keyfile, "global", "port", &error);
  if (port == 0 && error != NULL)
    {
      port = 110;
      g_error_free (error);
    }
  error = NULL;
  server_address = g_key_file_get_string (keyfile, "global", "server",
                                          &error);
  if (server_address == NULL && error != NULL)
    {
      server_address = g_strdup ("127.0.0.1");
      g_error_free (error);
    }

  inetaddr = gnet_inetaddr_new_nonblock (conf_address, port);
  if (gnet_server_new (inetaddr, port,
                       new_client, server_address) == NULL)
    {
      fprintf (stderr, "Could not create server.\n");
      exit (1);
    }

  g_message ("Listening at %s:%d.", conf_address, port);

  daemon (0, 0);

  g_free (conf_address);

  g_main_loop_run (g_main_loop_new (g_main_context_default (), TRUE));

  gnutls_global_deinit ();

  g_free (server_address);

  return 0;

}
